--
-- Farmet Kompaktomat K 1570
--
-- by Defender
--

Farmet_K1570 = {};

function Farmet_K1570.prerequisitesPresent(specializations)
    return true;
end;

function Farmet_K1570:load(xmlFile)
  print("Farmet_K1570 loading ...");
  _G.lng:loadTexts(self.baseDirectory.."modDesc.xml");
  self.workMode = false;
  self.turnOff = false;
  self.folding = false;
  self.unfolding = false;
  self.aiTerrainDetailChannel1 = g_currentMission.ploughChannel;
  self.aiTerrainDetailChannel2 = g_currentMission.sowingChannel;
  self.safeMode = true;
	self.AIon = false;
	self.broken = false;
	self.repairPrice = 1000;
	self.repairCount = 0;
	self.repairAgainPrice = 500;
	self.vehicleType = getXMLString(xmlFile, "vehicle#type");	
    
  local foldAnimRootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.unfoldAnim#rootNode"));
  self.foldAnimCharSet = 0;
  if foldAnimRootNode ~= nil and foldAnimRootNode ~= 0 then
    self.foldAnimCharSet = getAnimCharacterSet(foldAnimRootNode);
    if self.foldAnimCharSet ~= 0 then
      local clip = getAnimClipIndex(self.foldAnimCharSet, getXMLString(xmlFile, "vehicle.unfoldAnim#animationClip"));
      assignAnimTrackClip(self.foldAnimCharSet, 0, clip);
      setAnimTrackLoopState(self.foldAnimCharSet, 0, false);
      self.foldAnimSpeedScale = 1;
      self.foldAnimDuration = getAnimClipDuration(self.foldAnimCharSet, clip);
      end;
  end;
  
  local sLAnimRootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.supportLegAnim#rootNode"));
  self.sLAnimCharSet = 0;
  if sLAnimRootNode ~= nil and sLAnimRootNode ~= 0 then
    self.sLAnimCharSet = getAnimCharacterSet(sLAnimRootNode);
    if self.sLAnimCharSet ~= 0 then
      local clip1 = getAnimClipIndex(self.sLAnimCharSet, getXMLString(xmlFile, "vehicle.supportLegAnim#animationClip"));
      assignAnimTrackClip(self.sLAnimCharSet, 0, clip1);
      setAnimTrackLoopState(self.sLAnimCharSet, 0, false);
      self.sLAnimSpeedScale = 1;
      self.sLAnimDuration = getAnimClipDuration(self.sLAnimCharSet, clip1);
      end;
  end;
  
  local numSpinnerParts = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.SpinnerParts#count"), 0);
  self.SpinnerParts = {};
  for i=1, numSpinnerParts do
    local partnamei = string.format("vehicle.SpinnerParts.part" .. "%d", i);
    self.SpinnerParts[i] = {};
    self.SpinnerParts[i].index = Utils.indexToObject(self.components, getXMLString(xmlFile, partnamei .. "#index"));
    self.SpinnerParts[i].Scale = Utils.getNoNil(getXMLInt(xmlFile, partnamei .. "#rotScale"), 10);
  end;
  self.rotationSpeed = 0;
  self.rotationSpeedDownRate = 0;
  self.lastMovingDirection = 0;

  local workSound = getXMLString(xmlFile, "vehicle.workSound#file");
  if workSound ~= nil and workSound ~= "" then
    workSound = Utils.getFilename(workSound, self.baseDirectory); 
    self.workSound = createSample("workSound");
    self.workSoundEnabled = false;
    loadSample(self.workSound, workSound, false);
    self.workSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workSound#pitchOffset"), 1);
    self.workSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workSound#volume"), 1);
  end;
  
  self.dustParticleSystems = {};
  local dustParticleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.dustParticleSystems#count"), 0);
  for i=1, dustParticleSystemCount do
    local namei = string.format("vehicle.dustParticleSystems.part%d", i);
    local nodei = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
    Utils.loadParticleSystem(xmlFile, self.dustParticleSystems, namei, nodei, false, nil, self.baseDirectory)
  end;	
  
  self.saveAILM = self.aiLeftMarker;
  self.aiLeftMarker = nil;
  self.speedViolationMaxTime = 5000;
  self.speedViolationTimer = self.speedViolationMaxTime;
end;

function Farmet_K1570:delete()
	Utils.setEmittingState(self.dustParticleSystems, false)
	if self.workSoundEnabled then
		stopSample(self.workSound);
		self.workSoundEnabled = nil;
	end;	
end;

function Farmet_K1570:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
  self.broken = Utils.getNoNil(getXMLBool(xmlFile, key.."#broken"), false);
  self.repairCount = Utils.getNoNil(getXMLInt(xmlFile, key.."#repairCount"), 0);
  return BaseMission.VEHICLE_LOAD_OK;
end;

function Farmet_K1570:getSaveAttributesAndNodes(nodeIdent)
  local attributes = 'broken="'..tostring(self.broken)..'" repairCount="'..tostring(self.repairCount)..'"';
  return attributes, nil;
end;

function Farmet_K1570:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Farmet_K1570:keyEvent(unicode, sym, modifier, isDown)
end;

function Farmet_K1570:update(dt)
  if self.broken then
    if getUserInRange(self.rootNode, 6) then
      local price = self.repairPrice + (self.repairAgainPrice * self.repairCount);
--      g_currentMission:addHelpButtonText("Opravit ("..tostring(price*g_i18n.currencyFactor).." "..g_i18n:getText("Currency_symbol")..")", InputBinding.REPAIR_KOMPAKTOMAT);
      g_currentMission:addExtraPrintText(_G.lng:getText("klavesa").." O"..":              ".._G.lng:getText("repeair").." ("..tostring(price*g_i18n.currencyFactor).." "..g_i18n:getText("Currency_symbol")..")");      
      if g_currentMission.missionStats.money < price then
        g_currentMission:addExtraPrintText(_G.lng:getText("no_money"));
      else
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
          g_currentMission.missionStats.money = g_currentMission.missionStats.money - price;
          self.broken = false;
          self.repairCount = self.repairCount + 1; 
        end;
      end;
    end; 
  end;
  if self.attacherVehicle then
		if self.attacherVehicle.isAITractorActivated ~= nil then
			self.AIon = self.attacherVehicle.isAITractorActivated;
			if not self.AIon then
			  self.turnOff = false;
      end; 
		end; 	
    if self.broken then
      self.attacherVehicle:detachImplementByObject(self);
    end;  
    if self:getIsActiveForInput() then
      if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
        if not self.workMode and not self.unfolding then
          if getAnimTrackTime(self.foldAnimCharSet, 0) < 0.0 then
            setAnimTrackTime(self.foldAnimCharSet, 0, 0.0);
          end;
          setAnimTrackSpeedScale(self.foldAnimCharSet, 0, self.foldAnimSpeedScale);
          enableAnimTrack(self.foldAnimCharSet, 0);
          self.folding=false;
          self.unfolding=true;
        else
          local d = Utils.degToRad(0);
          setJointRotationLimit(self.componentJoints[2].jointIndex, 0, true, d, d);
          Utils.setEmittingState(self.dustParticleSystems, false)
          if self.workSoundEnabled then
            stopSample(self.workSound);
            self.workSoundEnabled = nil;
          end;        
          if getAnimTrackTime(self.foldAnimCharSet, 0) > self.foldAnimDuration then
            setAnimTrackTime(self.foldAnimCharSet, 0, self.foldAnimDuration);
          end;
          setAnimTrackSpeedScale(self.foldAnimCharSet, 0, -self.foldAnimSpeedScale);
          enableAnimTrack(self.foldAnimCharSet, 0);
          self.workMode = false;
          self.aiLeftMarker = nil;
          AITractor.updateToolsInfo(self.attacherVehicle);
          self.folding=true;
          self.unfolding=false;
        end;    
      end;
      if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
        self.safeMode = not self.safeMode;
      end;
    end;
    if self.unfolding and getAnimTrackTime(self.foldAnimCharSet, 0) >= self.foldAnimDuration then
      local a = Utils.degToRad(-3);
      local a1 = Utils.degToRad(3);
      setJointRotationLimit(self.componentJoints[2].jointIndex, 0, true, a, a1);
      self.unfolding = false;
      self.workMode = true;
      self.aiLeftMarker = self.saveAILM; 
      AITractor.updateToolsInfo(self.attacherVehicle);       
    end;
    if self.workMode then
      if not self.turnOff  and not self.broken then
        local cA;
        if self.AIon then
          cA = self.cuttingAreas[2];
        else
          cA = self.cuttingAreas[1];
        end;
        local x,y,z = getWorldTranslation(cA.start);
        local x1,y1,z1 = getWorldTranslation(cA.width);
        local x2,y2,z2 = getWorldTranslation(cA.height);
        if self.safeMode then
          Farmet_K1570.updateSafeArea(x, z, x1, z1, x2, z2)
        else
          Utils.updateCultivatorArea(x, z, x1, z1, x2, z2);      
        end;
      end;
      if self.attacherVehicle~=nil and self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed * 3600 > 20 then
        self.speedViolationTimer = self.speedViolationTimer - dt;
        if self.speedViolationTimer < 0 and self.attacherVehicle~=nil then
          self.attacherVehicle:detachImplementByObject(self);
          self.broken = true;
        end;
      else
        self.speedViolationTimer = self.speedViolationMaxTime;       
      end;
      
      if self.attacherVehicle ~= nil then
        self.rotationSpeed = self.attacherVehicle.lastSpeed * self.attacherVehicle.movingDirection;
      else
        self.rotationSpeed = 0;
      end;
      
      self.rotationSpeedDownRate = math.abs(self.rotationSpeed * 0.02);
      if self.attacherVehicle~=nil then 
        self.lastMovingDirection = self.attacherVehicle.movingDirection;
      end;
      self.rotationSpeed = self.rotationSpeed - self.rotationSpeedDownRate;
      local rotS = self.rotationSpeed * 1000; 
      if rotS == 0 or self.turnOff then
        if rotS == 0 then
          self.rotationSpeed = 0;
        end;
        Utils.setEmittingState(self.dustParticleSystems, false)
        if self.workSoundEnabled then
          stopSample(self.workSound);
          self.workSoundEnabled = nil;
        end;
      else
        Utils.setEmittingState(self.dustParticleSystems, true)
        if not self.workSoundEnabled and not self.AIon then
          setSamplePitch(self.workSound, self.workSoundPitchOffset);
          playSample(self.workSound, 0, self.workSoundVolume, 0);
          self.workSoundEnabled = true;
        end;
      end; 	
      for i=1, table.getn(self.SpinnerParts) do
        rotate(self.SpinnerParts[i].index , self.SpinnerParts[i].Scale * self.rotationSpeed ,0,0);
      end;
    end;
  end;
  if self.folding or self.unfolding then
    setJointFrame(self.componentJoints[2].jointIndex, 0,self.componentJoints[2].jointNode);
    setJointFrame(self.componentJoints[3].jointIndex, 0,self.componentJoints[3].jointNode);
    setJointFrame(self.componentJoints[4].jointIndex, 0,self.componentJoints[4].jointNode);
    setJointFrame(self.componentJoints[6].jointIndex, 0,self.componentJoints[6].jointNode);
    setJointFrame(self.componentJoints[8].jointIndex, 0,self.componentJoints[8].jointNode);
    local d = 0;
    local d1 = 0;
    local t = getAnimTrackTime(self.foldAnimCharSet, 0); 
    if t>9750 then
      d = -3;
      d1 = 3;
    elseif t>9500 then
      d = -2;
      d1 = 2;
    elseif t>9250 then
      d = -1;
      d1 = 1;
    elseif t>9000 then
      d = -0.5;
      d1 = 0.5;              
    end;
    local r = Utils.degToRad(d);
    local r1 = Utils.degToRad(d1);    
    for i=3, 8 do
      setJointRotationLimit(self.componentJoints[i].jointIndex, 2, true, r, r1);
    end;
  end;

  local tm = getAnimTrackTime(self.sLAnimCharSet, 0);
  if (tm>0) and (tm<self.sLAnimDuration) then
    local joint = self.componentJoints[1];
    setJointFrame(joint.jointIndex, 0,joint.jointNode);
  end;
  if self.folding and getAnimTrackTime(self.foldAnimCharSet, 0) <= 0 then
    self.folding = false;
  end;  
end;

function Farmet_K1570:draw()
  if self.workMode or self.unfolding then
--    g_currentMission:addHelpButtonText("Slozit Kompaktomat", InputBinding.IMPLEMENT_EXTRA2);
    g_currentMission:addExtraPrintText(_G.lng:getText("klavesa").." X"..":                   ".._G.lng:getText("fold_kompaktomat"));    
  else
--    g_currentMission:addHelpButtonText("Rozlozit Kompaktomat", InputBinding.IMPLEMENT_EXTRA2);
    g_currentMission:addExtraPrintText(_G.lng:getText("klavesa").." X"..":                   ".._G.lng:getText("unfold_kompaktomat"));    
  end;
  if self.safeMode then
--    g_currentMission:addHelpButtonText("Vypnout bezpecny mod", InputBinding.IMPLEMENT_EXTRA);
    g_currentMission:addExtraPrintText(_G.lng:getText("klavesa").." B"..":                   ".._G.lng:getText("turn_safe_off"));    
  else
--    g_currentMission:addHelpButtonText("Zapnout bezpecny mod", InputBinding.IMPLEMENT_EXTRA);
    g_currentMission:addExtraPrintText(_G.lng:getText("klavesa").." B"..":                   ".._G.lng:getText("turn_safe_on"));
  end;
  if math.abs(self.speedViolationTimer - self.speedViolationMaxTime) > 2 then
    g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "1", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL1)), 0.092, 0.048)
  end
  
end;

function Farmet_K1570:aiLower()
  if not self.safeMode then
    self.turnOff = false;
  end;
end; 


function Farmet_K1570:aiRaise()
  if not self.safeMode then
    self.turnOff = true;
  end;
end;

function Farmet_K1570:onDeactivate() 
  self.speedViolationTimer = self.speedViolationMaxTime;
  Utils.setEmittingState(self.dustParticleSystems, false)
  if self.workSoundEnabled then
    stopSample(self.workSound);
    self.workSoundEnabled = nil;
  end;  
end;

function Farmet_K1570:onDetach()
  self:onDeactivate();  
  if getAnimTrackTime(self.sLAnimCharSet, 0) > self.sLAnimDuration then
    setAnimTrackTime(self.sLAnimCharSet, 0, self.sLAnimDuration);
  end;
  setAnimTrackSpeedScale(self.sLAnimCharSet, 0, -self.sLAnimSpeedScale);
  enableAnimTrack(self.sLAnimCharSet, 0);  
end;

function Farmet_K1570:onAttach()
  if getAnimTrackTime(self.sLAnimCharSet, 0) < 0 then
    setAnimTrackTime(self.sLAnimCharSet, 0, 0.0);
  end;
  setAnimTrackSpeedScale(self.sLAnimCharSet, 0, self.sLAnimSpeedScale);
  enableAnimTrack(self.sLAnimCharSet, 0);    
end;

Farmet_K1570.updateSafeArea = function(l_18_0, l_18_1, l_18_2, l_18_3, l_18_4, l_18_5)
  local cultiId = g_currentMission.cultivatorChannel
  local sowingId = g_currentMission.sowingChannel
  local detailId = g_currentMission.terrainDetailId
  local ploughId = g_currentMission.ploughChannel
  local sprayId = g_currentMission.sprayChannel
  local x, z, widthX, widthZ, heightX, heightZ = Utils.getXZWidthAndHeight(detailId, l_18_0, l_18_1, l_18_2, l_18_3, l_18_4, l_18_5)
  Utils.updateDestroyCommonArea(l_18_0, l_18_1, l_18_2, l_18_3, l_18_4, l_18_5)
  setDensityMaskedParallelogram(detailId, x, z, widthX, widthZ, heightX, heightZ, cultiId, 1, detailId, sowingId, 1, 1)
  setDensityMaskedParallelogram(detailId, x, z, widthX, widthZ, heightX, heightZ, cultiId, 1, detailId, ploughId, 1, 1)
  setDensityParallelogram(detailId, x, z, widthX, widthZ, heightX, heightZ, sowingId, 1, 0)
  setDensityParallelogram(detailId, x, z, widthX, widthZ, heightX, heightZ, ploughId, 1, 0)
  setDensityParallelogram(detailId, x, z, widthX, widthZ, heightX, heightZ, sprayId, 1, 0)
end


function getUserInRange(node, minDistance)
  local result = false; 
  local px, py, pz = getWorldTranslation(node);
  local Pl = 0;
  for k,v in pairs(g_currentMission.players) do
    if g_currentMission.players[k].isControlled then
      Pl = g_currentMission.players[k].rootNode;
      break;
    end; 
  end;
  if Pl ~= 0 then
    local vx, vy, vz = getWorldTranslation(Pl);
    local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
    if distance < minDistance then		
      result = true;
    end;
  end;
  return result;
end;
